@@ -17,10 +17,10 @@
 #ifndef _BUTTON_H_
 #define _BUTTON_H_
 
-#include "node.h"
+#include "breaker.h"
 
-@interface Button: Node {
-    int index, state, selected;
+@interface Button: Breaker {
+    int index;
 }
 
 @end
