@@ -100,7 +100,7 @@ static void calculate_view_frustum()
 static void look_up_sample(int j, int i, float *h, float *e)
 {
     int a, b, c, d;
-	unsigned int k, l, m;
+    unsigned int k, l, m;
 
     a = context->size[0];
     b = context->size[1];
@@ -278,7 +278,7 @@ static void prioritize_diamond(Diamond *d)
     float e, w, h;
     float *C, p_w[3], p_c[3], dp_c[3], dp_s[2], p_0[3], p_1[3];
     int i;
-
+    
     e = d->error;
 
     p_w[0] = d->center[0];
@@ -806,28 +806,6 @@ static void reclassify_subtree(Triangle *n, int p)
     context->culled += 1;
 }
 
-static void test_subtree(Triangle *r)
-{
-    /* ... */
-    
-    if(!is_leaf(r)) {
-	test_subtree(r->children[0]);
-	test_subtree(r->children[1]);
-    }
-}
-
-static void create_levels(Triangle *r, int n)
-{
-    if(n > 0) {
-	if(is_leaf(r)) {
-	    split_triangle_pair(r);
-	}
-
-	create_levels(r->children[0], n - 1);
-	create_levels(r->children[1], n - 1);
-    }
-}
-
 static void draw_subtree(Triangle *n)
 {
     if(!is_out(n)) {
@@ -1156,6 +1134,18 @@ static void free_mesh()
     }
 }
 
+#if 0
+
+static void test_subtree(Triangle *r)
+{
+    /* ... */
+    
+    if(!is_leaf(r)) {
+	test_subtree(r->children[0]);
+	test_subtree(r->children[1]);
+    }
+}
+
 static int split_highest_pairs(lua_State *L)
 {
     Diamond *d;
@@ -1212,6 +1202,18 @@ static int merge_lowest_pairs(lua_State *L)
     return 0;
 }
 
+static void create_levels(Triangle *r, int n)
+{
+    if(n > 0) {
+	if(is_leaf(r)) {
+	    split_triangle_pair(r);
+	}
+
+	create_levels(r->children[0], n - 1);
+	create_levels(r->children[1], n - 1);
+    }
+}
+
 static int create_tree_level(lua_State *L)
 {
     int i, n;
@@ -1258,6 +1260,8 @@ static void traverse_quadtree(Triangle *n, Triangle *m, int l)
     }
 }
 
+#endif
+
 @implementation Elevation
 
 -(Elevation *) initWith: (int)width
@@ -1301,7 +1305,12 @@ static void traverse_quadtree(Triangle *n, Triangle *m, int l)
 
 -(Elevation *) init
 {
+    char *list[] = {
+	"detail", "orientation", "position", "target"
+    };
+
     [super init];
+    [self add: sizeof (list) / sizeof (char *) Properties: list];
 
     self->target = 5000;
    
@@ -1506,27 +1515,6 @@ static void traverse_quadtree(Triangle *n, Triangle *m, int l)
 	lua_pushnumber (_L, self->resolution[i]);
 	lua_rawseti (_L, -2, i + 4);
     }
-
-    lua_pushnumber (_L, self->triangles);
-    lua_rawseti (_L, -2, 7);
-
-    lua_pushnumber (_L, self->visible);
-    lua_rawseti (_L, -2, 8);
-
-    lua_pushnumber (_L, self->diamonds);
-    lua_rawseti (_L, -2, 9);
-
-    lua_pushnumber (_L, self->queued[0]);
-    lua_rawseti (_L, -2, 10);
-
-    lua_pushnumber (_L, self->queued[1]);
-    lua_rawseti (_L, -2, 11);
-
-    lua_pushnumber (_L, self->minimum);
-    lua_rawseti (_L, -2, 12);
-
-    lua_pushnumber (_L, self->maximum);
-    lua_rawseti (_L, -2, 13);
 }
 
 -(void) get
@@ -1537,6 +1525,29 @@ static void traverse_quadtree(Triangle *n, Triangle *m, int l)
 
     if (!xstrcmp(k, "target")) {
 	lua_pushnumber (_L, self->target);
+    } else if (!xstrcmp(k, "state")) {
+	lua_newtable (_L);
+
+	lua_pushnumber (_L, self->triangles);
+	lua_rawseti (_L, -2, 1);
+
+	lua_pushnumber (_L, self->visible);
+	lua_rawseti (_L, -2, 2);
+
+	lua_pushnumber (_L, self->diamonds);
+	lua_rawseti (_L, -2, 3);
+
+	lua_pushnumber (_L, self->queued[0]);
+	lua_rawseti (_L, -2, 4);
+
+	lua_pushnumber (_L, self->queued[1]);
+	lua_rawseti (_L, -2, 5);
+
+	lua_pushnumber (_L, self->minimum);
+	lua_rawseti (_L, -2, 6);
+
+	lua_pushnumber (_L, self->maximum);
+	lua_rawseti (_L, -2, 7);
     } else {
 	[super get];
     }
@@ -1558,47 +1569,97 @@ static void traverse_quadtree(Triangle *n, Triangle *m, int l)
 	
 	    lua_rawgeti (_L, 3, 1);
 
-	    length = lua_objlen (_L, -1);
-	    heights = (unsigned char *)malloc(length * sizeof (unsigned char));
+	    length = luaX_objlen (_L, -1);
 
-	    for(i = 0 ; i < length ; i += 1) {
-		lua_rawgeti(_L, -1, i + 1);
-		heights[i] = (unsigned char)(UCHAR_MAX * lua_tonumber(_L, -1));
-		lua_pop(_L, 1);
+	    if (!lua_getmetatable (_L, -1)) {
+		lua_newtable(_L);
+	    }
+
+	    lua_getfield (_L, -1, "unsigned char");
+
+	    if (lua_isstring (_L, -1)) {
+		heights = (unsigned char *)malloc(lua_strlen (_L, -1));
+		memcpy(heights, lua_tostring (_L, -1), lua_strlen (_L, -1));
+		
+		lua_pop (_L, 2);
+	    } else {
+		heights = (unsigned char *)malloc(length *
+						  sizeof (unsigned char));
+
+		for(i = 0 ; i < length ; i += 1) {
+		    lua_pushinteger (_L, i + 1);
+		    lua_gettable (_L, -2);
+		    heights[i] = (unsigned char)(UCHAR_MAX *
+						 lua_tonumber(_L, -1));
+		    lua_pop(_L, 1);
+		}
 	    }
 
 	    lua_pop (_L, 1);
 
 	    lua_rawgeti (_L, 3, 2);
 
-	    length = lua_objlen (_L, -1);
-	    errors = (unsigned short *)malloc(length * sizeof (unsigned short));
-
-	    for(i = 0 ; i < length ; i += 1) {
-		lua_rawgeti(_L, -1, i + 1);
-		errors[i] = (unsigned short)lua_tonumber(_L, -1);
-		lua_pop(_L, 1);
+	    if (!lua_getmetatable (_L, -1)) {
+		lua_newtable(_L);
 	    }
 
-	    lua_pop (_L, 1);
+	    lua_getfield (_L, -1, "unsigned short");
+
+	    if (lua_isstring (_L, -1)) {
+		errors = (unsigned short *)malloc(lua_strlen (_L, -1));
+		memcpy(errors, lua_tostring (_L, -1), lua_strlen (_L, -1));
+		
+		lua_pop (_L, 2);
+	    } else {
+		lua_pop (_L, 2);
+
+		errors = (unsigned short *)malloc(length *
+						  sizeof (unsigned short));
+
+		for(i = 0 ; i < length ; i += 1) {
+		    lua_pushinteger (_L, i + 1);
+		    lua_gettable (_L, -2);
+		    errors[i] = (unsigned short)lua_tonumber(_L, -1);
+		    lua_pop(_L, 1);
+		}
+	    }
 
 	    [self loadElevationTile: n
 	                withSamples: heights
 	                  andBounds: errors
 	                   ofLength: length];
+
+	    lua_pop (_L, 1);
 	    
 	    lua_rawgeti (_L, 3, 3);
 
-	    if (lua_istable (_L, 3)) {
-		length = lua_objlen (_L, -1);
-		pixels = (unsigned char *)malloc(length *
-						 sizeof (unsigned char));
+	    if (lua_istable (_L, -1)) {		
+		length = luaX_objlen (_L, -1);
 
-		for(i = 0 ; i < length ; i += 1) {
-		    lua_rawgeti(_L, -1, i + 1);
-		    pixels[i] = (unsigned char)(UCHAR_MAX *
-						lua_tonumber(_L, -1));
-		    lua_pop(_L, 1);
+		if (!lua_getmetatable (_L, -1)) {
+		    lua_newtable(_L);
+		}
+
+		lua_getfield (_L, -1, "unsigned char");
+
+		if (lua_isstring (_L, -1)) {
+		    pixels = (unsigned char *)malloc(lua_strlen (_L, -1));
+		    memcpy(pixels, lua_tostring (_L, -1), lua_strlen (_L, -1));
+
+		    lua_pop (_L, 2);
+		} else {
+		    lua_pop (_L, 2);
+
+		    pixels = (unsigned char *)malloc(length *
+						     sizeof (unsigned char));
+
+		    for(i = 0 ; i < length ; i += 1) {
+			lua_pushinteger (_L, i + 1);
+			lua_gettable (_L, -2);
+			pixels[i] = (unsigned char)(UCHAR_MAX *
+						    lua_tonumber(_L, -1));
+			lua_pop(_L, 1);
+		    }
 		}
 
 		[self loadImageryTile: n withPixels: pixels ofLength: length];
@@ -1611,16 +1672,35 @@ static void traverse_quadtree(Triangle *n, Triangle *m, int l)
 	    unsigned char *pixels;
 	    int length;
 
-	    length = lua_objlen (_L, -1);
-	    pixels = (unsigned char *)malloc(length * sizeof (unsigned char));
+	    length = luaX_objlen (_L, -1);
 
-	    for(i = 0 ; i < length ; i += 1) {
-		lua_rawgeti(_L, 3, i + 1);
-		pixels[i] = (unsigned char)(UCHAR_MAX * lua_tonumber(_L, -1));
-		lua_pop(_L, 1);
+	    if (!lua_getmetatable (_L, -1)) {
+		lua_newtable(_L);
+	    }
+
+	    lua_getfield (_L, -1, "unsigned char");
+
+	    if (lua_isstring (_L, -1)) {
+		pixels = (unsigned char *)malloc(lua_strlen (_L, -1));
+		memcpy(pixels, lua_tostring (_L, -1), lua_strlen (_L, -1));
+
+		lua_pop (_L, 2);
+	    } else {
+		lua_pop (_L, 2);
+
+		pixels = (unsigned char *)malloc(length * sizeof (unsigned char));
+
+		for(i = 0 ; i < length ; i += 1) {
+		    lua_pushinteger (_L, i + 1);
+		    lua_gettable (_L, -2);
+		    pixels[i] = (unsigned char)(UCHAR_MAX * lua_tonumber(_L, -1));
+		    lua_pop(_L, 1);
+		}
 	    }
 
 	    [self loadDetailPixels: pixels ofLength: length];
+
+	    lua_pop(_L, 1);
 	}	
     } else if (!xstrcmp(k, "target")) {
 	self->target = lua_tonumber (_L, -1);
@@ -1664,20 +1744,30 @@ static void traverse_quadtree(Triangle *n, Triangle *m, int l)
 -(void) traversePass: (int)pass
 {
     if (pass == 1) {
+	glEnable (GL_DEPTH_TEST);
+	glEnable (GL_CULL_FACE);
+
 	glMatrixMode (GL_MODELVIEW);
 	glPushMatrix();
-	glMultMatrixf ([self homogenous]);
+	glMultMatrixd ([self matrix]);
 
 	glScalef(self->resolution[0],
 		 self->resolution[1],
 		 self->resolution[2]);
 
+	/* glPolygonMode (GL_FRONT_AND_BACK, GL_LINE); */
+
 	switch_to_context(self);
 	optimize_earth_geometry();
 	draw_earth_geometry();
 
+	/* glPolygonMode (GL_FRONT_AND_BACK, GL_FILL); */
+
 	glMatrixMode (GL_MODELVIEW);
 	glPopMatrix();
+
+	glDisable (GL_DEPTH_TEST);
+	glDisable (GL_CULL_FACE);
     }
 
     [super traversePass: pass];
