@@ -17,13 +17,10 @@
 #ifndef _EVENT_H_
 #define _EVENT_H_
 
-#define N 64
-
-#include <GL/glx.h>
 #include "node.h"
 
 @interface Event: Node {
-    int buttonpress, buttonrelease, motion;
+    int buttonpress, buttonrelease, motion, scroll;
     int keypress, keyrelease;
 }
 @end
