@@ -18,7 +18,7 @@
 #include <lauxlib.h>
 #include <GL/gl.h>
 
-#include "particulate.h"
+#include "meteorology.h"
 #include "fluid.h"
 
 #define STRIDE 11
@@ -65,7 +65,14 @@ static unsigned int hash(int i, int j, int k, unsigned int n)
 
 -(Fluid *) init
 {
+    char *list[] = {
+	"clustering", "density", "diffusion", "granularity", "hashes",
+	"mobility", "scale", "size", "state", "stiffness", "support",
+	"viscosity"
+    };
+
     self = [super init];
+    [self add: sizeof (list) / sizeof (char *) Properties: list];
 
     self->scale[0] = 1;
     self->scale[1] = 1;
@@ -178,7 +185,7 @@ static unsigned int hash(int i, int j, int k, unsigned int n)
 	    double mu_0, mu_1, mu_2;
 	    double sigma_0, sigma_1, sigma_2;
 	    double x, y, z, u, v, w;
-	    float *R, *r;
+	    double *R, *r;
 
 	    R = [self rotation];
 	    r = [self translation];
@@ -360,7 +367,7 @@ static unsigned int hash(int i, int j, int k, unsigned int n)
     }
 }
 
--(void)cleanup
+-(void)finish
 {
     int i;
 
@@ -390,10 +397,10 @@ static unsigned int hash(int i, int j, int k, unsigned int n)
     self->state[0] /= self->generated;
     self->state[1] /= self->generated;
    
-    [super cleanup];
+    [super finish];
 }
 
--(void)stepBy: (double)dt
+-(void)stepBy: (double) dt at: (double) t_0
 {
     double rho_0, k_g, mu, k_s, mu_s, sigma, h, d, D, K_1, K_2;
     int i, j, s, t, q;
@@ -596,13 +603,10 @@ static unsigned int hash(int i, int j, int k, unsigned int n)
 		           r_ij[2] * r_ij[2];
 
 		if (sigma > 0) {
-		    double r[3];
-		    
-		    r[0] = r_i[0] + r_ij[0];
-		    r[1] = r_i[1] + r_ij[1];
-		    r[2] = r_i[2] + r_ij[2];
-
-		    get_turbulence_at (r, v);
+		    get_turbulence_at (r_i[0] + r_ij[0],
+				       r_i[1] + r_ij[1],
+				       r_i[2] + r_ij[2],
+				       t_0, v);
 		}
 
 		/* Accumulate the subparticle pressure, viscosity
@@ -624,7 +628,7 @@ static unsigned int hash(int i, int j, int k, unsigned int n)
 	}
     }
 
-    [super stepBy: dt];
+    [super stepBy: dt at: t_0];
 
     self->iteration += 1;
 }
@@ -652,7 +656,8 @@ static unsigned int hash(int i, int j, int k, unsigned int n)
 
 	    glPushMatrix();
 	    glTranslated (r_i[0], r_i[1], r_i[2]);
-	
+	    glEnable (GL_DEPTH_TEST);
+ 	
 	    glBegin (GL_QUADS);
 	
 	    for (j = 0 ; j < self->clustering ; j += 1) {
@@ -692,6 +697,8 @@ static unsigned int hash(int i, int j, int k, unsigned int n)
 	    }
 
 	    glEnd();
+	    
+	    glDisable (GL_DEPTH_TEST);
 	    glPopMatrix();
 	}
     }
