@@ -21,8 +21,8 @@
 #include "widget.h"
 
 @interface Gauge: Widget {
-    int range[2], spacing[2];
-    float spread[2], thickness, reading, background[4];
+    int range[2], spacing[2], size;
+    double radius[2], spread[2], thickness, background[4], *reading;
 }
 
 @end
