@@ -19,16 +19,17 @@
 
 #include <lua.h>
 #include <ode/ode.h>
-#include <objc/Object.h>
+#include "node.h"
 
-@interface Joint: Object {
+@interface Joint: Node {
     dJointID joint;
     dJointFeedback feedback;
+
+    int attach, inverted, debug;
 }
 
 -(dJointID) joint;
--(void) get;
--(void) set;
+-(void) update;
 
 @end
 
