@@ -19,11 +19,11 @@
 
 #include <lua.h>
 #include <ode/ode.h>
-#include "node.h"
+#include "transform.h"
 
-@interface Piston: Node {
+@interface Piston: Transform {
     double output, speed, diameter, inertia;
-    double throttle, rating[2], ratio;
+    double throttle, bypass, idle, rating[2], ratio;
     
     struct {
 	int length;
